//
//  MARSRestFul.m
//  RESTFul
//
//  Created by Nhat Huy on 8/20/13.
//  Copyright (c) 2013 Mulodo Viet Nam. All rights reserved.
//

#import "MARSRestFul.h"

@interface MARSRestFul ()

@property (nonatomic,strong) NSURLRequest *urlRequest;
@property (nonatomic,strong) NSURLConnection *urlConnection;
@property (nonatomic,strong) NSURLResponse *urlResponse;
@property (nonatomic,strong) NSDictionary *parameters;

@property (nonatomic,copy) Complete completionBlock;
@property (nonatomic,copy) Error errorBlock;

@end

@implementation MARSRestFul

+ (id)sharedRESTFul {
    
    static MARSRestFul *restFul;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        
        restFul = [[self alloc] init];
    });
    
    return restFul;
}

- (id)init {
    self = [super init];
    if (self) {
        
        _parameters = [[NSDictionary alloc] init];
    }
    return self;
}

- (void)postPayload:(NSDictionary *)payload apiName:(NSString *)apiName complete:(Complete)completionBlock didFail:(Error)errorBlock {
    
    _apiName = apiName;
    _parameters = payload;
    _completionBlock = completionBlock;
    _errorBlock = errorBlock;
    
    [self setUpRequest];
    [self sendRequest];
    
    NSLog(@"In here");
}

- (void)sendRequest {
    
    _urlConnection = [[NSURLConnection alloc] initWithRequest:_urlRequest delegate:self];
    if (_urlConnection) {
        
        _data = [[NSMutableData alloc] init];
    }
}

- (void)setUpRequest {
    
    NSMutableURLRequest *urlRequest = [[NSMutableURLRequest alloc] init];
    
    NSString *stringUrl = [NSString stringWithFormat:@"%@%@", kApiUrl, _apiName];
    
    [urlRequest setURL:[NSURL URLWithString:stringUrl]];
    [urlRequest setHTTPMethod:@"POST"];
    
    // This is important! //NSURLConnection is very sensitive to format.
    NSString *boundary = @"29154787378233593420402431"; 
    
    // Use multipart/form-data to upload file
    NSString *contentType = [NSString stringWithFormat:@"multipart/form-data; boundary=%@",boundary];
    [urlRequest addValue:contentType forHTTPHeaderField: @"Content-Type"];
    
    // Request body
    NSMutableData *body = [NSMutableData data];
    [body appendData:[[NSString stringWithFormat:@"--%@\r\n",boundary] dataUsingEncoding:NSUTF8StringEncoding]];
    
    // Add parameter into body of request
    for (NSString *key in [_parameters allKeys]) {
        
        [body appendData:[[NSString stringWithFormat:@"Content-Disposition: form-data; name=\"%@\"\r\n\r\n", key] dataUsingEncoding:NSUTF8StringEncoding]];
        [body appendData:[[_parameters objectForKey:key] dataUsingEncoding:NSUTF8StringEncoding]];
        [body appendData:[[NSString stringWithFormat:@"\r\n--%@\r\n",boundary] dataUsingEncoding:NSUTF8StringEncoding]];
    }
    
    [body appendData:[[NSString stringWithFormat:@"\r\n--%@--\r\n",boundary] dataUsingEncoding:NSUTF8StringEncoding]];
    // setting the body of the post to the request
    [urlRequest setHTTPBody:body];
    
    _urlRequest = urlRequest;
}

#pragma mark - URLConnection Delegate

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
    
    _urlResponse = response;
    [_data setLength:0];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    
    [_data appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
    
    _errorBlock(error);
    _errorBlock = NULL;
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
    
    NSLog(@"Finish");
    _completionBlock(_data, _urlResponse);
    _completionBlock = NULL;
}

@end
